import React from 'react';
type RenderItem = React.FunctionComponent<React.HTMLProps<HTMLDivElement> & {
    index: number;
    ['data-vlist-item-idx']: number;
}>;
type RenderHeader = React.FunctionComponent;
type RenderEmptyList = React.FunctionComponent;
type VirtualGridProps = {
    /**
     * Grid item minimum width (items containers will autostretch to the grid
     * container size)
     */
    itemMinWidth: number;
    /**
     * Grid row height
     */
    itemHeight: number;
    /**
     * Number of items in the grid
     */
    itemsCount: number;
    /**
     * Number of columns (default is fitting as much as possible on the screen
     * based on itemMinWidth)
     */
    colCount?: number;
    /**
     * Render method for grid cell content. Receives props that should be passed
     * directly to the cell root node for the keyboard navigation to work
     * correctly
     */
    renderItem: RenderItem;
    /**
     * Custom grid item key (default is item index)
     */
    itemKey?: (index: number) => React.Key | null | undefined;
    /**
     * Header content height
     */
    headerHeight?: number;
    /**
     * Render method for grid header component. This should be used if header
     * content should be scrollable as part of the whole grid
     */
    renderHeader?: RenderHeader;
    /**
     * Render method for empty list items.
     */
    renderEmptyList?: RenderEmptyList;
    /**
     * Number of rows rendered off-screen (default: 3)
     */
    overscanCount?: number;
    /**
     * Custom styling for individual elements within the grid DOM.
     */
    classNames?: {
        container?: string;
        header?: string;
        grid?: string;
        row?: string;
        cell?: string;
    };
    /**
     * Set to `false` of you want the last focused item to be preserved between
     * focus / blur (default: true)
     */
    resetActiveItemOnBlur?: boolean;
};
export declare const VirtualGrid: React.ForwardRefExoticComponent<Omit<VirtualGridProps & Omit<React.HTMLProps<HTMLDivElement>, keyof VirtualGridProps>, "ref"> & React.RefAttributes<HTMLDivElement>>;
export {};
//# sourceMappingURL=virtual-grid.d.ts.map